@@ -4,7 +4,6 @@ import (
 	"bytes"
 	"context"
 	"errors"
-	"io"
 	"io/ioutil"
 	"os"
 	"path/filepath"
@@ -60,9 +59,7 @@ func (s *WorktreeSuite) TestPullCheckout(c *C) {
 }
 
 func (s *WorktreeSuite) TestPullFastForward(c *C) {
-	url, clean := s.TemporalDir()
-	defer clean()
-
+	url := c.MkDir()
 	path := fixtures.Basic().ByTag("worktree").One().Worktree().Root()
 
 	server, err := PlainClone(url, false, &CloneOptions{
@@ -70,10 +67,7 @@ func (s *WorktreeSuite) TestPullFastForward(c *C) {
 	})
 	c.Assert(err, IsNil)
 
-	dir, clean := s.TemporalDir()
-	defer clean()
-
-	r, err := PlainClone(dir, false, &CloneOptions{
+	r, err := PlainClone(c.MkDir(), false, &CloneOptions{
 		URL: url,
 	})
 	c.Assert(err, IsNil)
@@ -97,9 +91,7 @@ func (s *WorktreeSuite) TestPullFastForward(c *C) {
 }
 
 func (s *WorktreeSuite) TestPullNonFastForward(c *C) {
-	url, clean := s.TemporalDir()
-	defer clean()
-
+	url := c.MkDir()
 	path := fixtures.Basic().ByTag("worktree").One().Worktree().Root()
 
 	server, err := PlainClone(url, false, &CloneOptions{
@@ -107,10 +99,7 @@ func (s *WorktreeSuite) TestPullNonFastForward(c *C) {
 	})
 	c.Assert(err, IsNil)
 
-	dir, clean := s.TemporalDir()
-	defer clean()
-
-	r, err := PlainClone(dir, false, &CloneOptions{
+	r, err := PlainClone(c.MkDir(), false, &CloneOptions{
 		URL: url,
 	})
 	c.Assert(err, IsNil)
@@ -217,8 +206,9 @@ func (s *WorktreeSuite) TestPullProgressWithRecursion(c *C) {
 
 	path := fixtures.ByTag("submodule").One().Worktree().Root()
 
-	dir, clean := s.TemporalDir()
-	defer clean()
+	dir, err := ioutil.TempDir("", "plain-clone-submodule")
+	c.Assert(err, IsNil)
+	defer os.RemoveAll(dir)
 
 	r, _ := PlainInit(dir, false)
 	r.CreateRemote(&config.RemoteConfig{
@@ -389,8 +379,9 @@ func (s *WorktreeSuite) TestCheckoutSymlink(c *C) {
 		c.Skip("git doesn't support symlinks by default in windows")
 	}
 
-	dir, clean := s.TemporalDir()
-	defer clean()
+	dir, err := ioutil.TempDir("", "checkout")
+	c.Assert(err, IsNil)
+	defer os.RemoveAll(dir)
 
 	r, err := PlainInit(dir, false)
 	c.Assert(err, IsNil)
@@ -422,9 +413,7 @@ func (s *WorktreeSuite) TestFilenameNormalization(c *C) {
 		c.Skip("windows paths may contain non utf-8 sequences")
 	}
 
-	url, clean := s.TemporalDir()
-	defer clean()
-
+	url := c.MkDir()
 	path := fixtures.Basic().ByTag("worktree").One().Worktree().Root()
 
 	server, err := PlainClone(url, false, &CloneOptions{
@@ -530,62 +519,6 @@ func (s *WorktreeSuite) TestCheckoutSubmoduleInitialized(c *C) {
 	c.Assert(status.IsClean(), Equals, true)
 }
 
-func (s *WorktreeSuite) TestCheckoutRelativePathSubmoduleInitialized(c *C) {
-	url := "https://github.com/git-fixtures/submodule.git"
-	r := s.NewRepository(fixtures.ByURL(url).One())
-
-	// modify the .gitmodules from original one
-	file, err := r.wt.OpenFile(".gitmodules", os.O_WRONLY|os.O_TRUNC, 0666)
-	c.Assert(err, IsNil)
-
-	n, err := io.WriteString(file, `[submodule "basic"]
-	path = basic
-	url = ../basic.git
-[submodule "itself"]
-	path = itself
-	url = ../submodule.git`)
-	c.Assert(err, IsNil)
-	c.Assert(n, Not(Equals), 0)
-
-	w, err := r.Worktree()
-	c.Assert(err, IsNil)
-
-	w.Add(".gitmodules")
-	w.Commit("test", &CommitOptions{})
-
-	// test submodule path
-	modules, err := w.readGitmodulesFile()
-
-	c.Assert(modules.Submodules["basic"].URL, Equals, "../basic.git")
-	c.Assert(modules.Submodules["itself"].URL, Equals, "../submodule.git")
-
-	basicSubmodule, err := w.Submodule("basic")
-	c.Assert(err, IsNil)
-	basicRepo, err := basicSubmodule.Repository()
-	c.Assert(err, IsNil)
-	basicRemotes, err := basicRepo.Remotes()
-	c.Assert(err, IsNil)
-	c.Assert(basicRemotes[0].Config().URLs[0], Equals, "https://github.com/git-fixtures/basic.git")
-
-	itselfSubmodule, err := w.Submodule("itself")
-	c.Assert(err, IsNil)
-	itselfRepo, err := itselfSubmodule.Repository()
-	c.Assert(err, IsNil)
-	itselfRemotes, err := itselfRepo.Remotes()
-	c.Assert(err, IsNil)
-	c.Assert(itselfRemotes[0].Config().URLs[0], Equals, "https://github.com/git-fixtures/submodule.git")
-
-	sub, err := w.Submodules()
-	c.Assert(err, IsNil)
-
-	err = sub.Update(&SubmoduleUpdateOptions{Init: true, RecurseSubmodules: DefaultSubmoduleRecursionDepth})
-	c.Assert(err, IsNil)
-
-	status, err := w.Status()
-	c.Assert(err, IsNil)
-	c.Assert(status.IsClean(), Equals, true)
-}
-
 func (s *WorktreeSuite) TestCheckoutIndexMem(c *C) {
 	fs := memfs.New()
 	w := &Worktree{
@@ -614,15 +547,17 @@ func (s *WorktreeSuite) TestCheckoutIndexMem(c *C) {
 }
 
 func (s *WorktreeSuite) TestCheckoutIndexOS(c *C) {
-	fs, clean := s.TemporalFilesystem()
-	defer clean()
+	dir, err := ioutil.TempDir("", "checkout")
+	c.Assert(err, IsNil)
+	defer os.RemoveAll(dir)
 
+	fs := osfs.New(filepath.Join(dir, "worktree"))
 	w := &Worktree{
 		r:          s.Repository,
 		Filesystem: fs,
 	}
 
-	err := w.Checkout(&CheckoutOptions{})
+	err = w.Checkout(&CheckoutOptions{})
 	c.Assert(err, IsNil)
 
 	idx, err := s.Repository.Storer.Index()
@@ -1037,15 +972,17 @@ func (s *WorktreeSuite) TestStatusAfterCheckout(c *C) {
 }
 
 func (s *WorktreeSuite) TestStatusModified(c *C) {
-	fs, clean := s.TemporalFilesystem()
-	defer clean()
+	dir, err := ioutil.TempDir("", "status")
+	c.Assert(err, IsNil)
+	defer os.RemoveAll(dir)
 
+	fs := osfs.New(filepath.Join(dir, "worktree"))
 	w := &Worktree{
 		r:          s.Repository,
 		Filesystem: fs,
 	}
 
-	err := w.Checkout(&CheckoutOptions{})
+	err = w.Checkout(&CheckoutOptions{})
 	c.Assert(err, IsNil)
 
 	f, err := fs.Create(".gitignore")
@@ -1129,15 +1066,17 @@ func (s *WorktreeSuite) TestStatusUntracked(c *C) {
 }
 
 func (s *WorktreeSuite) TestStatusDeleted(c *C) {
-	fs, clean := s.TemporalFilesystem()
-	defer clean()
+	dir, err := ioutil.TempDir("", "status")
+	c.Assert(err, IsNil)
+	defer os.RemoveAll(dir)
 
+	fs := osfs.New(filepath.Join(dir, "worktree"))
 	w := &Worktree{
 		r:          s.Repository,
 		Filesystem: fs,
 	}
 
-	err := w.Checkout(&CheckoutOptions{})
+	err = w.Checkout(&CheckoutOptions{})
 	c.Assert(err, IsNil)
 
 	err = fs.Remove(".gitignore")
@@ -1367,8 +1306,9 @@ func (s *WorktreeSuite) TestAddRemoved(c *C) {
 }
 
 func (s *WorktreeSuite) TestAddSymlink(c *C) {
-	dir, clean := s.TemporalDir()
-	defer clean()
+	dir, err := ioutil.TempDir("", "checkout")
+	c.Assert(err, IsNil)
+	defer os.RemoveAll(dir)
 
 	r, err := PlainInit(dir, false)
 	c.Assert(err, IsNil)
@@ -1829,39 +1769,7 @@ func (s *WorktreeSuite) TestClean(c *C) {
 	// An empty dir should be deleted, as well.
 	_, err = fs.Lstat("pkgA")
 	c.Assert(err, ErrorMatches, ".*(no such file or directory.*|.*file does not exist)*.")
-}
 
-func (s *WorktreeSuite) TestCleanBare(c *C) {
-	storer := memory.NewStorage()
-
-	r, err := Init(storer, nil)
-	c.Assert(err, IsNil)
-	c.Assert(r, NotNil)
-
-	wtfs := memfs.New()
-
-	err = wtfs.MkdirAll("worktree", os.ModePerm)
-	c.Assert(err, IsNil)
-
-	wtfs, err = wtfs.Chroot("worktree")
-	c.Assert(err, IsNil)
-
-	r, err = Open(storer, wtfs)
-	c.Assert(err, IsNil)
-
-	wt, err := r.Worktree()
-	c.Assert(err, IsNil)
-
-	_, err = wt.Filesystem.Lstat(".")
-	c.Assert(err, IsNil)
-
-	// Clean with Dir: true.
-	err = wt.Clean(&CleanOptions{Dir: true})
-	c.Assert(err, IsNil)
-
-	// Root worktree directory must remain after cleaning
-	_, err = wt.Filesystem.Lstat(".")
-	c.Assert(err, IsNil)
 }
 
 func (s *WorktreeSuite) TestAlternatesRepo(c *C) {
@@ -2081,11 +1989,7 @@ func (s *WorktreeSuite) TestGrep(c *C) {
 	}
 
 	path := fixtures.Basic().ByTag("worktree").One().Worktree().Root()
-
-	dir, clean := s.TemporalDir()
-	defer clean()
-
-	server, err := PlainClone(dir, false, &CloneOptions{
+	server, err := PlainClone(c.MkDir(), false, &CloneOptions{
 		URL: path,
 	})
 	c.Assert(err, IsNil)
@@ -2134,8 +2038,9 @@ func (s *WorktreeSuite) TestGrep(c *C) {
 }
 
 func (s *WorktreeSuite) TestAddAndCommit(c *C) {
-	dir, clean := s.TemporalDir()
-	defer clean()
+	dir, err := ioutil.TempDir("", "plain-repo")
+	c.Assert(err, IsNil)
+	defer os.RemoveAll(dir)
 
 	repo, err := PlainInit(dir, false)
 	c.Assert(err, IsNil)
